#ifndef ZP_PARSER_H
#define ZP_PARSER_H

#include <iostream>
#include <vector>
#include <memory>
#include <stdexcept>

namespace zp
{
    enum class Instruction : char;

    struct Block;
    struct InstructionBlock;
    struct UnionBlock;
    struct Loop;

    // forward declaration
    class AstVisitor;

    class Parser;
    class ParserException;
}


enum class zp::Instruction : char
{
    PLUS,
    MINUS,
    LEFT,
    RIGHT,
    DOT,
    COMMA
};


struct zp::Block
{
    virtual ~Block(void) = default;

    virtual void accept(AstVisitor& v) = 0;
};


struct zp::InstructionBlock :
    public Block
{
    std::vector<Instruction> instructions;

    inline void addInstruction(Instruction i)               { instructions.push_back(i); }
    inline bool isEmpty(void) const                         { return instructions.empty(); }

    void accept(AstVisitor& v);
};


struct zp::UnionBlock :
    public Block
{
    std::vector<std::unique_ptr<Block>> instructions;
    inline void addBlock(std::unique_ptr<Block>&& block)    { instructions.push_back(std::move(block)); }
    
    void accept(AstVisitor& v);
};


struct zp::Loop :
    public UnionBlock
{
    void accept(AstVisitor& v);
};


class zp::Parser
{
    std::istream& in;
public:
    Parser(std::istream& in);

    std::unique_ptr<Block> parse(void);
private:
    std::unique_ptr<Block> parseBlock(bool);
};


class zp::ParserException :
    public std::runtime_error
{
public:
    inline ParserException(const std::string& what) : runtime_error(what) {}
};

#endif /* ZP_PARSER_H */

